/*!
 * \file archiveMaker.hpp
 * \brief This file is used create archive file.
 * Copyright (C) 2011-2013 Nippon Telegraph and Telephone Corporation
 *
 * [License]
 *
 * $id$
 */

#ifndef _ARCHIVE_MAKER_H
#define _ARCHIVE_MAKER_H

#include <limits.h>
#include "util.hpp"

/*!
 * \brief This class create archive file.
 */
class TArchiveMaker {
  public:
    /*!
     * \brief TArchiveMaker constructor.
     */
    TArchiveMaker(void);
    /*!
     * \brief TArchiveMaker destructor.
     */
    virtual ~TArchiveMaker(void);
    
    /*!
     * \brief Setting target file path.
     * \param targetPath [in] Path of archive target file.
     */
    void setTarget(char const* targetPath);
    
    /*!
     * \brief Do file archive and create archive file.
     * \param env         [in] JNI environment object.
     * \param archiveFile [in] archive file name.
     * \return Response code of execute commad line.
     */
    virtual int doArchive(JNIEnv* env, char const* archiveFile) = 0;
    
    /*!
     * \brief Clear archive file data.
     */
    void clear(void);
    
  protected:
    /*!
     * \brief Getting target file path.
     * \return Path of archive target file.
     */
    char const *getTarget(void);
    
  private:
    /*!
     * \brief Path of archive source file.
     */
    char sourcePath[PATH_MAX + 1];
    
};

#endif // _ARCHIVE_MAKER_H
