/*
 * ObjectData.java
 * Created on 2011/10/13
 * RCSID: $Id:$
 *
 * Copyright (C) 2011-2013 Nippon Telegraph and Telephone Corporation
 *
 */

package jp.co.ntt.oss.heapstats.container;

import java.io.Serializable;

/**
 * implements {@link Serializable}, {@link Cloneable} <br>
 * <br>
 * Contains information about the classes loaded in the java virtual machine.
 */
public class ObjectData implements
                       Serializable,
                       Cloneable {
    /** serialVersionUID. */
    private static final long serialVersionUID = -1685786847473848152L;

    /** Class Tag. */
    private long tag;

    /** Class Name. */
    private String name;

    /** Instances of classes. */
    private long count;

    /** Total heap usage class. */
    private long totalSize;

    /**
     * Create a ObjectData.
     */
    public ObjectData() {
        tag = 0;
        name = null;
        count = 0;
        totalSize = 0;
    }

    /**
     * Sets the tag.
     *
     * @param value tag
     */
    public final void setTag(final long value) {
        tag = value;
    }

    /**
     * Return the tag.
     *
     * @return Return the tag
     */
    public final long getTag() {
        return tag;
    }

    /**
     * Sets the Class path.
     *
     * @param value Class path
     */
    public final void setName(final String value) {
        name = value;
    }

    /**
     * Return the Class Name.
     *
     * @return Return the Class Name
     */
    public final String getName() {
        return name;
    }

    /**
     * Sets the Object count.
     *
     * @param value Number of Classes
     */
    public final void setCount(final long value) {
        count = value;
    }

    /**
     * Return the Object count.
     *
     * @return Return the Object count
     */
    public final long getCount() {
        return count;
    }

    /**
     * Sets the total heap size.
     *
     * @param value total heap size
     */
    public final void setTotalSize(final long value) {
        totalSize = value;
    }

    /**
     * Return the total heap size.
     *
     * @return Return the total heap size
     */
    public final long getTotalSize() {
        return totalSize;
    }

    @Override
    public final String toString() {
        StringBuffer buf = new StringBuffer();

        buf.append("0x");
        buf.append(Long.toHexString(tag));
        buf.append(", ");
        buf.append(name);
        buf.append(", ");
        buf.append(count);
        buf.append(", ");
        buf.append(totalSize);

        return buf.toString();
    }

    @Override
    public final ObjectData clone() {
        try {
            super.clone();
        } catch (CloneNotSupportedException e) {
            e.printStackTrace();
        }

        ObjectData cloneObj = new ObjectData();

        cloneObj.setTag(tag);
        cloneObj.setName(name);
        cloneObj.setCount(count);
        cloneObj.setTotalSize(totalSize);

        return cloneObj;
    }
}
