/*
 * FindDialog.java
 * Created on 2012/05/24
 * RCSID: $Id: FindDialog.java 440 2012-07-19 09:03:01Z alpha_takahiro.jyoukou $
 *
 * Copyright (C) 2011-2013 Nippon Telegraph and Telephone Corporation
 *
 */

package jp.co.ntt.oss.heapstats.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;

/**
 * extends {@link JDialog}<br>
 * implements {@link ActionListener}<br>
 * <br>
 * Search the contents of the log details.
 */
public class FindDialog extends JDialog implements
                                       ActionListener {
    /** serialVersionUID. */
    private static final long serialVersionUID = 2750747037965338897L;

    /** Text area that displays a detailed log. */
    private JTextArea textArea;

    /** Enter a search string. */
    private JComboBox word;

    /** Button is pressed, to conduct the search towards the end. */
    private JButton next;

    /** Close the dialog box to cancel the search. */
    private JButton cancel;

    /**
     * Creates a modeless dialog with the specified Dialog as its owner.
     *
     * @param parent the owner Dialog from which the dialog is displayed or null
     *        if this dialog has no owner
     * @param area Text area that displays a detailed log
     */
    public FindDialog(final JDialog parent, final JTextArea area) {
        super(parent);
        setTitle(GuiCommon.getResources().getString(GuiCommon.FIND_TITLE));
        setSize(400, 90);
        setResizable(false);
        setLocationRelativeTo(parent);

        this.textArea = area;

        addComponent();
    }

    @Override
    public final void actionPerformed(final ActionEvent e) {
        Object eventSource = e.getSource();

        if (eventSource.equals(next)) {
            if (word.getSelectedIndex() < 0) {
                word.addItem(word.getSelectedItem());
            }
            find();
        } else if (eventSource.equals(cancel)) {
            this.setVisible(false);
        }
    }

    /**
     * Add the component to the screen.
     */
    private void addComponent() {
        JPanel main = new JPanel();
        main.setLayout(new BorderLayout(GuiCommon.BORDER_SIZE,
                GuiCommon.BORDER_SIZE));
        add(main);
        JLabel dammy = new JLabel("");
        dammy.setBackground(Color.RED);

        JPanel whatPanel = new JPanel();
        whatPanel.setLayout(new BorderLayout(GuiCommon.BORDER_SIZE,
                GuiCommon.BORDER_SIZE));
        JLabel what = new JLabel(GuiCommon.getResources().getString(
                GuiCommon.FIND_LABEL)
                + " : ");
        whatPanel.add(what, BorderLayout.NORTH);

        JPanel field = new JPanel();
        field.setLayout(new BorderLayout(GuiCommon.BORDER_SIZE,
                GuiCommon.BORDER_SIZE));

        word = new JComboBox();
        word.setEditable(true);
        field.add(word, BorderLayout.NORTH);

        JPanel btn = new JPanel();
        btn.setLayout(new BorderLayout(GuiCommon.BORDER_SIZE,
                GuiCommon.BORDER_SIZE));
        next = new JButton(GuiCommon.getResources().getString(
                GuiCommon.FIND_NEXT));
        cancel = new JButton(GuiCommon.getResources().getString(
                GuiCommon.FIND_CANCEL));
        next.addActionListener(this);
        cancel.addActionListener(this);
        btn.add(next, BorderLayout.NORTH);
        btn.add(cancel, BorderLayout.SOUTH);

        main.add(dammy, BorderLayout.NORTH);
        main.add(whatPanel, BorderLayout.WEST);
        main.add(field, BorderLayout.CENTER);
        main.add(btn, BorderLayout.EAST);
    }

    /**
     * Conduct the search of the specified string.
     */
    private void find() {
        Object inputWord = word.getSelectedItem();
        if (textArea.getText().isEmpty() || inputWord == null) {
            return;
        }

        int start = textArea.getText().indexOf(
                inputWord.toString(), textArea.getSelectionEnd());
        if (start != -1) {
            textArea.setFocusable(true);
            textArea.setSelectionStart(start);
            textArea.setSelectionEnd(start
                    + word.getSelectedItem().toString().length());
        } else {
            GuiCommon.showMessage(this, GuiCommon.BASEI03, word
                    .getSelectedItem().toString(),
                    JOptionPane.INFORMATION_MESSAGE);
        }
    }
}
