/*
 * LogDetailDialog.java
 * Created on 2012/01/24
 * RCSID: $Id: LogDetailDialog.java 437 2012-07-19 08:39:41Z alpha_takahiro.jyoukou $
 *
 * Copyright (C) 2011-2013 Nippon Telegraph and Telephone Corporation
 *
 */
package jp.co.ntt.oss.heapstats.gui;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;
import javax.swing.text.DefaultEditorKit;

import jp.co.ntt.oss.heapstats.container.ThreadData;

/**
 * extends {@link JDialog}<br>
 * implements {@link ActionListener}<br>
 * <br>
 * Draw a graphical representation of system resources and java.
 */
public class LogDetailDialog extends JDialog implements
                                            ActionListener {
    /** serialVersionUID. */
    private static final long serialVersionUID = -1796436599399658541L;

    /** Number of columns in the log collection list. */
    private static final int COL_COUNT = 4;

    private static final int COL_IDX_PATH = 3;

    /** The width of the item column. */
    private static final int ITEM_WIDTH = 100;

    /** The width of the data column. */
    private static final int DATA_WIDTH = 130;

    /** The width of the browse button. */
    private static final int BROWSE_WIDTH = 35;

    /** Hexadecimal. */
    private static final int HEX_STARING = 16;

    /** Socket status:ESTABLISHED. */
    private static final int VAL_ESTABLISHED = 1;

    /** Socket status:SYN_SENT. */
    private static final int VAL_SYN_SENT = 2;

    /** Socket status:SYN_RECV. */
    private static final int VAL_SYN_RECV = 3;

    /** Socket status:FIN_WAIT1. */
    private static final int VAL_FIN_WAIT1 = 4;

    /** Socket status:FIN_WAIT2. */
    private static final int VAL_FIN_WAIT2 = 5;

    /** Socket status:TIME_WAIT. */
    private static final int VAL_TIME_WAIT = 6;

    /** Socket status:CLOSE. */
    private static final int VAL_CLOSE = 7;

    /** Socket status:CLOSE_WAIT. */
    private static final int VAL_CLOSE_WAIT = 8;

    /** Socket status:LAST_ACK. */
    private static final int VAL_LAST_ACK = 9;

    /** Socket status:LISTEN. */
    private static final int VAL_LISTEN = 10;

    /** Socket status:CLOSING. */
    private static final int VAL_CLOSING = 11;

    /** Socket endpoints header string. */
    private static final String HEAD_PROTO = "Proto ";

    /** Socket endpoints header string. */
    private static final String HEAD_RECV_Q = "Recv-Q ";

    /** Socket endpoints header string. */
    private static final String HEAD_SEND_Q = "Send-Q ";

    /** Socket endpoints header string. */
    private static final String HEAD_V4_LOCAL = "Local Address               ";

    /** Socket endpoints header string. */
    private static final String HEAD_V6_LOCAL = "Local Address                                       ";

    /** Socket endpoints header string. */
    private static final String HEAD_V4_FOREIGN = "Foreign Address             ";

    /** Socket endpoints header string. */
    private static final String HEAD_V6_FOREIGN = "Foreign Address                                   ";

    /** Socket endpoints header string. */
    private static final String HEAD_STATE = "State";

    /** Summary header of deadlock. */
    private static final String HEAD_THREAD_CLASS = "class";

    /** Summary header of deadlock. */
    private static final String HEAD_THREAD_METHOD = "method";

    /** Summary header of deadlock. */
    private static final String HEAD_THREAD_THREAD = "waiter-thread";

    /** Summary header of deadlock. */
    private static final String HEAD_THREAD_WAIT = "owner-thread";

    /** Socket status:ESTABLISHED. */
    private static final String ESTABLISHED = "ESTABLISHED";

    /** Socket status:SYN_SENT. */
    private static final String SYN_SENT = "SYN_SENT";

    /** Socket status:FIN_WAIT1. */
    private static final String SYN_RECV = "SYN_RECV";

    /** Socket status:FIN_WAIT1. */
    private static final String FIN_WAIT1 = "FIN_WAIT1";

    /** Socket status:FIN_WAIT2. */
    private static final String FIN_WAIT2 = "FIN_WAIT2";

    /** Socket status:TIME_WAIT. */
    private static final String TIME_WAIT = "TIME_WAIT";

    /** Socket status:CLOSE. */
    private static final String CLOSE = "CLOSE";

    /** Socket status:CLOSE_WAIT. */
    private static final String CLOSE_WAIT = "CLOSE_WAIT";

    /** Socket status:LAST_ACK. */
    private static final String LAST_ACK = "LAST_ACK";

    /** Socket status:LISTEN. */
    private static final String LISTEN = "LISTEN";

    /** Socket status:CLOSING. */
    private static final String CLOSING = "CLOSING";

    /** Trigger the collection log is resource exhausted. */
    private static final int TRIGGER_RESOURCE_EXHAUSTED = 1;

    /** Trigger the collection log is signal. */
    private static final int TRIGGER_SIGNAL = 2;

    /** Trigger the collection log is dead lock. */
    private static final int TRIGGER_DEADLOCK = 4;

    /** Font size of the display area is a detailed log. */
    private static final int FONT_SIZE = 15;

    /** Item name indicates the socket end point of view a detailed log. */
    private static final String SOCKET_ITEMNAME = "Socket";

    /** Item name display shows the detailed log of deadlock. */
    private static final String DEADLOCK_ITEMNAME = "ThreadDump";

    /** The name of the file at run-time environment information. */
    private static final String FILE_ENVINFO = "envInfo.txt";

    /** The zip file lists failure information. */
    private List<String> zipList;

    /** Mainframe heapstats. */
    private GUIMain guimain;

    /** Main Component. */
    private JSplitPane comp;

    /** Component of the Next button. */
    private JButton nextBtn;

    /** Previous Button Component. */
    private JButton preBtn;

    /** Table model to display the details of the log. */
    private DefaultTableModel model;

    /** All information display area. */
    private JTextArea detail;

    /** Index of zip files that are currently displayed. */
    private int currentIndex;

    /** Find dialog. */
    private FindDialog findDlg;

    /** Pop-up menu. */
    private JPopupMenu popup;

    /** Find menu in the pop-up menu. */
    private JMenuItem search;

    /** Holds the type of factors that the agent was logged. */
    private int trigger;

    /**
     * Creates a modeless dialog with the specified Dialog as its owner.
     *
     * @param owner the owner Dialog from which the dialog is displayed or null
     *        if this dialog has no owner
     */
    public LogDetailDialog(final GUIMain owner) {
        guimain = owner;

        setTitle(GuiCommon.getResources().getString(GuiCommon.TAB_TITLE_LOG));
        setSize(GuiCommon.DEFAULT_DIALOG_WIDTH, GuiCommon.DEFAULT_DIALOG_HEIGHT);
        setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        setModal(false);

        comp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, null, null);
        addComponent();
        createPopupMenu();
        setLocationByPlatform(true);

        findDlg = new FindDialog(this, detail);
    }

    /**
     * Return the component.
     *
     * @return Return the Component
     */
    public final JSplitPane getComp() {
        return comp;
    }

    /**
     * Sets the dialog component.
     *
     * @param value the Main Component
     */
    public final void setComp(final JSplitPane value) {
        comp = value;
        add(comp);
    }

    /**
     * Clear the log Detail data.
     */
    public final void clear() {
        model.setRowCount(0);
        detail.setText("");
        nextBtn.setEnabled(false);
        preBtn.setEnabled(false);
    }

    /**
     * Starts drawing the LogDetail Data.
     *
     * @param fileList The zip file lists failure information.
     * @param index The index lists the zip file to decode
     */
    public final void draw(final List<String> fileList, final int index) {
        zipList = fileList;
        currentIndex = index;
        if (currentIndex > 0) {
            preBtn.setEnabled(true);
        } else {
            preBtn.setEnabled(false);
        }
        if (currentIndex < fileList.size() - 1) {
            nextBtn.setEnabled(true);
        } else {
            nextBtn.setEnabled(false);
        }

        if (zipList.size() > 0) {
            File zip = new File(zipList.get(index));
            int dot = zip.getName().indexOf(".");
            String dirName;
            if (dot > 0) {
                dirName = zip.getName().substring(0, dot);
            } else {
                dirName = zip.getName();
            }

            File decParent = new File(zip.getParent(), dirName);

            File decdir = unzip(zip, decParent, true);

            if (decdir != null) {
                if (!readEnvInfo(decdir) || !readLogFiles(decdir)
                        || !readSocketEndpoint(decdir)
                        || !readThreadDump(decdir)) {
                    model.setRowCount(0);
                    detail.setText("");
                    unzip(zip, decParent, false);
                    readEnvInfo(decdir);
                    readLogFiles(decdir);
                    readSocketEndpoint(decdir);
                    readThreadDump(decdir);
                }
            }
        }

        detail.setFont(new Font(Font.MONOSPACED, Font.PLAIN, FONT_SIZE));
        comp.revalidate();
    }

    @Override
    public final void actionPerformed(final ActionEvent actionevent) {
        Object eventSource = actionevent.getSource();

        if (eventSource.equals(nextBtn)) {
            model.setRowCount(0);
            detail.setText("");
            draw(zipList, currentIndex + 1);
        } else if (eventSource.equals(preBtn)) {
            model.setRowCount(0);
            detail.setText("");
            draw(zipList, currentIndex - 1);
        } else if (eventSource.equals(search)) {
            if (!findDlg.isAlwaysOnTop()) {
                findDlg.setAlwaysOnTop(true);
            }
            findDlg.setVisible(true);
        }
    }

    /**
     * Create components to draw on the screen.
     */
    private void addComponent() {
        String[] columnNames = new String[COL_COUNT];
        columnNames[0] = GuiCommon.getResources().getString(
                GuiCommon.LOG_HEAD_ITEM);
        columnNames[1] = GuiCommon.getResources().getString(
                GuiCommon.LOG_HEAD_DATA);
        columnNames[2] = "";
        columnNames[COL_IDX_PATH] = "";
        model = new DefaultTableModel(columnNames, 0);

        model.setRowCount(0);

        JTable itemEntries = new JTable(model);

        detail = new JTextArea();
        JScrollPane textScr = new JScrollPane(detail);
        BrowseButton browse = new BrowseButton(this, itemEntries, detail);

        itemEntries.getColumnModel().getColumn(0).setPreferredWidth(ITEM_WIDTH);
        itemEntries.getColumnModel().getColumn(1).setPreferredWidth(DATA_WIDTH);
        TableColumn column = itemEntries.getColumnModel().getColumn(2);
        column.setCellRenderer(browse);
        column.setCellEditor(browse);
        column.setMinWidth(BROWSE_WIDTH);
        column.setMaxWidth(BROWSE_WIDTH);

        column = itemEntries.getColumnModel().getColumn(COL_IDX_PATH);
        column.setMinWidth(0);
        column.setMaxWidth(0);

        JScrollPane itemScr = new JScrollPane(itemEntries);
        itemScr.setPreferredSize(new Dimension(ITEM_WIDTH + DATA_WIDTH
                + BROWSE_WIDTH, itemScr.getHeight()));

        preBtn = new JButton(GuiCommon.getResources().getString(
                GuiCommon.PREV_BUTTON));
        preBtn.setEnabled(false);
        preBtn.addActionListener(this);
        nextBtn = new JButton(GuiCommon.getResources().getString(
                GuiCommon.NEXT_BUTTON));
        nextBtn.setEnabled(false);
        nextBtn.addActionListener(this);

        JPanel nextPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
        nextPanel.add(preBtn);
        nextPanel.add(new JLabel("     "));
        nextPanel.add(nextBtn);
        JSplitPane itemSplit = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
                nextPanel, itemScr);
        itemSplit.setDividerSize(0);

        comp.setLeftComponent(itemSplit);
        comp.setRightComponent(textScr);
        comp.setOneTouchExpandable(true);
    }

    /**
     * Creates the popup menu.
     */
    private void createPopupMenu() {
        popup = new JPopupMenu();

        JMenuItem cut = new JMenuItem(GuiCommon.getResources().getString(
                GuiCommon.POPUP_CUT), KeyEvent.VK_T);
        cut.addActionListener(new DefaultEditorKit.CutAction());
        popup.add(cut);

        JMenuItem copy = new JMenuItem(GuiCommon.getResources().getString(
                GuiCommon.POPUP_COPY), KeyEvent.VK_C);
        copy.addActionListener(new DefaultEditorKit.CopyAction());
        popup.add(copy);

        JMenuItem paste = new JMenuItem(GuiCommon.getResources().getString(
                GuiCommon.POPUP_PASTE), KeyEvent.VK_P);
        paste.addActionListener(new DefaultEditorKit.PasteAction());
        popup.add(paste);

        popup.addSeparator();

        search = new JMenuItem(GuiCommon.getResources().getString(
                GuiCommon.POPUP_FIND), KeyEvent.VK_F);
        search.addActionListener(this);
        popup.add(search);

        detail.setComponentPopupMenu(popup);
    }

    /**
     * Unzip the zip archive.
     *
     * @param file a zip archive
     * @param parentDir The parent directory unzipped
     * @param first Unzip the first flag true:first false:Second
     * @return Return the unzipped directory
     */
    private File unzip(final File file, final File parentDir,
            final boolean first) {
        try {
            ZipFile zipFile = new ZipFile(file);
            File decodeDir = parentDir;
            Enumeration<? extends ZipEntry> entries = zipFile.entries();
            while (entries.hasMoreElements()) {
                ZipEntry entry = entries.nextElement();
                File entryFile = new File(parentDir, entry.getName());
                if (entry.isDirectory()) {
                    decodeDir = entryFile;
                    if (!entryFile.exists()) {
                        entryFile.mkdirs();
                    }
                    continue;
                }

                if (!parentDir.exists()) {
                    decodeDir.mkdirs();
                }

                if (entryFile.exists()) {
                    continue;
                }

                BufferedInputStream inBuff = new BufferedInputStream(
                        zipFile.getInputStream(entry));
                BufferedOutputStream outBuff = new BufferedOutputStream(
                        new FileOutputStream(entryFile));

                int buffSize = inBuff.available();
                while (buffSize > 0) {
                    byte[] buff = new byte[buffSize];
                    inBuff.read(buff);
                    outBuff.write(buff);
                    buffSize = inBuff.available();
                }

                inBuff.close();
                outBuff.close();
            }

            zipFile.close();

            Object[] row = new Object[COL_COUNT];
            row[0] = "FileName";
            row[1] = file.getName();
            model.addRow(row);

            return decodeDir;
        } catch (Throwable e) {
            GuiCommon.showMessage(guimain, GuiCommon.BASEE19, file.getName(),
                    JOptionPane.ERROR_MESSAGE);
            e.printStackTrace();
        }
        return null;
    }

    /**
     * Reads envInfo.txt, to set a table.
     *
     * @param decDir Unzipped directory
     * @return Return the result reads
     */
    private boolean readEnvInfo(final File decDir) {
        BufferedReader read = null;
        try {
            File file = new File(decDir, FILE_ENVINFO);
            if (!file.exists()) {
                return false;
            }
            read = new BufferedReader(new FileReader(file));

            String line = read.readLine();
            int i = 0;
            while (line != null) {
                String[] row = new String[2];
                String[] tmp = line.split("=");

                row[0] = tmp[0];
                switch (i) {
                case 0:
                    // Collection date
                    SimpleDateFormat sdf = new SimpleDateFormat(
                            GuiCommon.DATE_FORMAT);
                    row[1] = sdf.format(Long.parseLong(tmp[1]));
                    break;
                case 1:
                    // Log Trigger
                    if (tmp[1].equals("1")) {
                        row[1] = "ResourceExhausted";
                        trigger = TRIGGER_RESOURCE_EXHAUSTED;
                    } else if (tmp[1].equals("2")) {
                        row[1] = "Signal";
                        trigger = TRIGGER_SIGNAL;
                    } else if (tmp[1].equals("4")) {
                        row[1] = "Deadlock";
                        trigger = TRIGGER_DEADLOCK;
                    } else {
                        row[1] = "N/A";
                    }
                    break;
                default:
                    if (tmp.length > 1) {
                        StringBuilder sb = new StringBuilder();
                        for (int n = 1; n < tmp.length; n++) {
                            sb.append(tmp[n]);
                            sb.append("=");
                        }
                        row[1] = sb.substring(0, sb.length() - 1);
                    } else {
                        row[1] = "N/A";
                    }
                    break;
                }

                model.addRow(row);
                line = read.readLine();
                i++;
            }
            return true;
        } catch (Throwable e) {
            GuiCommon.showMessage(guimain, GuiCommon.BASEE08, FILE_ENVINFO,
                    JOptionPane.ERROR_MESSAGE);
            clear();
            e.printStackTrace();
        } finally {
            try {
                if (read != null) {
                    read.close();
                }
            } catch (IOException e) {
                GuiCommon.showMessage(guimain, GuiCommon.BASEE08, FILE_ENVINFO,
                        JOptionPane.ERROR_MESSAGE);
                clear();
                e.printStackTrace();
            }
        }
        return false;
    }

    /**
     * Read the log file that is set in the properties file.
     *
     * @param decDir Unzipped directory
     * @return Return the result reads
     */
    private boolean readLogFiles(final File decDir) {
        BufferedReader read = null;
        boolean ret = true;
        try {
            if (GuiCommon.getLogfiles() == null) {
                return ret;
            }

            for (String view : GuiCommon.getLogfiles().split(",")) {
                if (view.length() == 0) {
                    ret = false;
                    continue;
                }
                File file = new File(decDir, view);

                Object[] row = new Object[COL_COUNT];
                row[0] = file.getName();
                if (!file.exists()) {
                    row[1] = "N/A";
                    model.addRow(row);
                    ret = false;
                    continue;
                }
                read = new BufferedReader(new FileReader(file));

                String line = read.readLine();
                StringBuilder sb = new StringBuilder();
                sb.append(line);
                line = read.readLine();
                if (line != null) {
                    sb.append("...");
                }

                row[1] = sb.toString();
                row[COL_IDX_PATH] = file.getAbsolutePath();

                model.addRow(row);

                read.close();
            }
            return ret;
        } catch (Throwable e) {
            GuiCommon.showMessage(guimain, GuiCommon.BASEE08,
                    GuiCommon.getLogfiles(), JOptionPane.ERROR_MESSAGE);
            ret = false;
            clear();
        } finally {
            try {
                if (read != null) {
                    read.close();
                }
            } catch (IOException e) {
                GuiCommon.showMessage(guimain, GuiCommon.BASEE08,
                        GuiCommon.getLogfiles(), JOptionPane.ERROR_MESSAGE);
                ret = false;
                e.printStackTrace();
            }
        }

        return ret;
    }

    /**
     * Read the socket file that is set in the properties file.
     *
     * @param decDir Unzipped directory
     * @return Return the result reads
     */
    private boolean readSocketEndpoint(final File decDir) {
        BufferedReader read = null;
        BufferedWriter write = null;
        boolean ret = false;

        try {
            File sockFile = new File(decDir, SOCKET_ITEMNAME);
            Object[] row = new Object[COL_COUNT];
            row[0] = SOCKET_ITEMNAME;

            if (GuiCommon.getSocketend() == null) {
                return !ret;
            }

            for (String name : GuiCommon.getSocketend().split(",")) {
                File file = new File(decDir, name);
                if (!file.exists() || name.length() == 0) {
                    continue;
                }

                if (write == null) {
                    write = new BufferedWriter(new FileWriter(sockFile));
                }

                read = new BufferedReader(new FileReader(file));
                read.readLine();

                if (!file.getName().endsWith("6")) {
                    // IPv4
                    writeSocket(write, read, file.getName(), true);
                } else {
                    // IPv6
                    writeSocket(write, read, file.getName(), false);
                }

                read.close();
                ret = true;
            }
            if (write != null) {
                write.flush();
            }

            if (ret) {
                row[1] = HEAD_PROTO + "...";
                row[COL_IDX_PATH] = sockFile.getAbsolutePath();
                model.addRow(row);
            }

            return ret;
        } catch (Throwable e) {
            GuiCommon.showMessage(guimain, GuiCommon.BASEE08,
                    GuiCommon.getSocketend(), JOptionPane.ERROR_MESSAGE);
            ret = false;
            e.printStackTrace();
        } finally {
            try {
                if (read != null) {
                    read.close();
                }
                if (write != null) {
                    write.close();
                }
            } catch (IOException e) {
                GuiCommon.showMessage(guimain, GuiCommon.BASEE08,
                        GuiCommon.getSocketend(), JOptionPane.ERROR_MESSAGE);
                ret = false;
                clear();
                e.printStackTrace();
            }
        }
        return ret;
    }

    /**
     * Extracts information from files deadlock the thread dump.
     *
     * @param decDir The directory that contains the thread dump file
     * @return Return the result of extraction
     */
    private boolean readThreadDump(final File decDir) {
        boolean ret = true;

        if (trigger != TRIGGER_DEADLOCK) {
            return ret;
        }

        Object[] row = new Object[COL_COUNT];
        row[0] = DEADLOCK_ITEMNAME;

        File dumpFile = new File(decDir, GuiCommon.THREAD_DUNM_FILE);
        File lockFile = new File(decDir, DEADLOCK_ITEMNAME);

        if (!dumpFile.exists()) {
            return ret;
        }

        row[1] = "[Summary] ...";
        row[COL_IDX_PATH] = lockFile.getAbsolutePath()
                + GuiCommon.SPLIT_CHARACTER + dumpFile.getAbsolutePath();

        if (lockFile.exists()) {
            model.addRow(row);
            return ret;
        }

        BufferedReader read = null;
        BufferedWriter write = null;
        try {
            read = new BufferedReader(new FileReader(dumpFile));
            String line = read.readLine();

            boolean isJvmti = false;
            ThreadData thread = null;
            Map<String, ThreadData> tmpMap = new HashMap<String, ThreadData>();

            while (line != null) {
                // Name of its own thread
                int classIndex = line.indexOf("\"");
                if (classIndex != -1) {
                    thread = new ThreadData();
                    thread.setThreadName(line.substring(classIndex + 1,
                            line.indexOf("\"", classIndex + 1)));
                }

                // Thread state
                if (thread != null
                        && line.indexOf("java.lang.Thread.State") != -1) {
                    String[] split = line.trim().split(" ");
                    if (!split[1].trim().equals("BLOCKED")) {
                        thread = null;
                    }
                }

                // Class name Method name
                if (thread != null && (line.trim().indexOf("at") == 0)) {
                    String[] split = line.trim().split(" ");
                    String classMethod = split[1].split("\\(")[0];
                    thread.setMethodName(classMethod.substring(classMethod
                            .lastIndexOf(".") + 1));
                    thread.setClassName(classMethod.substring(0,
                            classMethod.lastIndexOf(".")));
                }

                // wait Address or name thread
                if (thread != null && line.indexOf("- waiting to lock") != -1) {
                    if (isJvmti || line.indexOf("owner:") != -1) {
                        // jvmti
                        isJvmti = true;
                        thread.setWaitThread(line.substring(
                                line.indexOf(":") + 1, line.indexOf(">")));
                    } else {
                        // socket
                        thread.setWaitLock(line.substring(
                                line.indexOf("<") + 1, line.indexOf(">")));
                    }
                }
                // lockObject address or threadName
                if (thread != null && line.indexOf("- locked") != -1) {
                    if (isJvmti) {
                        // jvmti sets the thread name
                        thread.setLockThread(thread.getThreadName());
                    } else {
                        // socket address
                        thread.setLockThread(line.substring(
                                line.indexOf("<") + 1, line.indexOf(">")));
                    }
                    tmpMap.put(thread.getLockThread(), thread);
                    thread = null;
                }

                if (thread != null
                        && line.indexOf("Found one Java-level deadlock") != -1) {
                    break;
                }
                line = read.readLine();
            }

            write = new BufferedWriter(new FileWriter(lockFile));

            if (!tmpMap.isEmpty()) {
                writeThreadDump(tmpMap, write, isJvmti);

                model.addRow(row);
            }
        } catch (IOException e) {
            GuiCommon.showMessage(guimain, GuiCommon.BASEE08,
                    dumpFile.getName(), JOptionPane.ERROR_MESSAGE);
            ret = false;
            e.printStackTrace();
        } catch (Throwable e) {
            GuiCommon.showMessage(guimain, GuiCommon.BASEE08,
                    dumpFile.getName(), JOptionPane.ERROR_MESSAGE);
            ret = false;
            e.printStackTrace();
        } finally {
            try {
                if (read != null) {
                    read.close();
                }
                if (write != null) {
                    write.close();
                }
            } catch (IOException e) {
                GuiCommon.showMessage(guimain, GuiCommon.BASEE08,
                        dumpFile.getName(), JOptionPane.ERROR_MESSAGE);
                ret = false;
                clear();
                e.printStackTrace();
            }
        }

        return ret;
    }

    /**
     * Create a file that shows the status of each protocol sockets.
     *
     * @param write The socket file buffer for writing
     * @param read Socket endpoints file(tcp,udp,tcp6,udp6)
     * @param name Protocol name
     * @param v4 IP version flag true:IPv4, false:IPv6
     * @throws IOException If an I/O error occurs
     */
    private void writeSocket(final BufferedWriter write,
            final BufferedReader read, final String name, final boolean v4)
            throws IOException {
        write.write(HEAD_PROTO);
        write.write(HEAD_RECV_Q);
        write.write(HEAD_SEND_Q);
        if (v4) {
            write.write(HEAD_V4_LOCAL);
            write.write(HEAD_V4_FOREIGN);
        } else {
            write.write(HEAD_V6_LOCAL);
            write.write(HEAD_V6_FOREIGN);
        }
        write.write(HEAD_STATE);

        write.newLine();
        String line = read.readLine();
        while (line != null) {
            write.write(name);
            writeBlank(write, HEAD_PROTO.length(), name.length());

            String[] data = line.trim().split(" ");

            // Recv_Q
            String[] tmp = data[4].split(":");
            String queue = Integer.toHexString(Integer.parseInt(tmp[1],
                    HEX_STARING));
            write.write(queue);
            writeBlank(write, HEAD_RECV_Q.length(), queue.length());

            // Send_Q
            queue = Integer.toHexString(Integer.parseInt(tmp[0], HEX_STARING));
            write.write(queue);
            writeBlank(write, HEAD_SEND_Q.length(), queue.length());

            // Local Address
            tmp = data[1].split(":");
            StringBuilder buf = new StringBuilder();
            if (v4) {
                buf.append(hexstringToIPv4(tmp[0]));
            } else {
                buf.append(hexstringToIPv6(tmp[0]));
            }
            buf.append(":");
            buf.append(Integer.parseInt(tmp[1], HEX_STARING));
            write.write(buf.toString());
            if (v4) {
                writeBlank(write, HEAD_V4_LOCAL.length(), buf.length());
            } else {
                writeBlank(write, HEAD_V6_LOCAL.length(), buf.length());
            }

            // Foreign Address
            tmp = data[2].split(":");
            buf.setLength(0);
            if (v4) {
                buf.append(hexstringToIPv4(tmp[0]));
            } else {
                buf.append(hexstringToIPv6(tmp[0]));
            }
            buf.append(":");
            buf.append(Integer.parseInt(tmp[1], HEX_STARING));
            write.write(buf.toString());
            if (v4) {
                writeBlank(write, HEAD_V4_FOREIGN.length(), buf.length());
            } else {
                writeBlank(write, HEAD_V6_FOREIGN.length(), buf.length());
            }

            // connection state
            writeState(write, Integer.parseInt(data[3], HEX_STARING));

            write.newLine();

            line = read.readLine();
        }
        write.newLine();
    }

    /**
     * Output a summary of the deadlock.
     *
     * @param threadMap Information of the thread that caused the deadlock
     * @param write The thread dump file buffer for writing
     * @param isJvmti If true, the output type of the thread dump file has been
     *        using the jvmti
     * @throws IOException If an I/O error occurs
     */
    private void writeThreadDump(final Map<String, ThreadData> threadMap,
            final BufferedWriter write, final boolean isJvmti)
            throws IOException {

        int classLength = HEAD_THREAD_CLASS.length() + 2;
        int methodLength = HEAD_THREAD_METHOD.length() + 2;
        int threadLength = HEAD_THREAD_THREAD.length() + 2;
        for (Map.Entry<String, ThreadData> entry : threadMap.entrySet()) {
            int tmp = entry.getValue().getClassName().length();
            if (classLength < tmp) {
                classLength = tmp + 2;
            }
            tmp = entry.getValue().getMethodName().length();
            if (methodLength < tmp) {
                methodLength = tmp + 2;
            }
            tmp = entry.getValue().getThreadName().length();
            if (threadLength < tmp) {
                threadLength = tmp + 2;
            }
        }

        write.write("[Summary]");
        write.newLine();

        write.write(HEAD_THREAD_CLASS);
        writeBlank(write, classLength, HEAD_THREAD_CLASS.length());
        write.write(HEAD_THREAD_METHOD);
        writeBlank(write, methodLength, HEAD_THREAD_METHOD.length());
        write.write(HEAD_THREAD_THREAD);
        writeBlank(write, threadLength, HEAD_THREAD_THREAD.length());
        write.write(HEAD_THREAD_WAIT);
        write.newLine();

        for (Map.Entry<String, ThreadData> entry : threadMap.entrySet()) {
            // class Name
            write.write(entry.getValue().getClassName());
            writeBlank(write, classLength, entry.getValue().getClassName()
                    .length());

            // method Name
            write.write(entry.getValue().getMethodName());
            writeBlank(write, methodLength, entry.getValue().getMethodName()
                    .length());

            // thread
            write.write(entry.getValue().getThreadName());
            writeBlank(write, threadLength, entry.getValue().getThreadName()
                    .length());

            // wait-thread
            if (isJvmti) {
                write.write(entry.getValue().getWaitThread());
            } else {
                write.write(threadMap.get(entry.getValue().getWaitLock())
                        .getThreadName());
            }
            write.newLine();
        }

        write.newLine();
        write.write("[Detail]");
        write.newLine();
    }

    /**
     * Setting up a blank to fill the specified size.
     *
     * @param write The socket file buffer for writing
     * @param length Header length of a single item
     * @param strLen The length of a character string to write in
     * @throws IOException If an I/O error occurs
     */
    private void writeBlank(final BufferedWriter write, final int length,
            final int strLen) throws IOException {
        for (int i = 0; i < (length - strLen); i++) {
            write.write(" ");
        }
    }

    /**
     * That converts decimal IPv4 of the IPv4 in hex.
     *
     * @param data IPv4 before conversion
     * @return The IPv4 of the converted
     */
    private String hexstringToIPv4(final String data) {

        StringBuilder ipv4 = new StringBuilder();
        try {
            ipv4.append(Integer.parseInt(data.substring(6, 8), HEX_STARING));
            ipv4.append(".");
            ipv4.append(Integer.parseInt(data.substring(4, 6), HEX_STARING));
            ipv4.append(".");
            ipv4.append(Integer.parseInt(data.substring(2, 4), HEX_STARING));
            ipv4.append(".");
            ipv4.append(Integer.parseInt(data.substring(0, 2), HEX_STARING));
        } catch (NumberFormatException e) {
            ipv4.setLength(0);
        }

        return ipv4.toString();
    }

    /**
     * That converts decimal IPv6 of the IPv6 in hex.
     *
     * @param data IPv6 before conversion
     * @return The IPv6 of the converted
     */
    private String hexstringToIPv6(final String data) {
        StringBuilder ipv6 = new StringBuilder();

        String hex = data.substring(6, 8) + data.substring(4, 6);
        ipv6.append(Integer.toHexString(Integer.parseInt(hex, HEX_STARING)));
        ipv6.append(":");

        hex = data.substring(2, 4) + data.substring(0, 2);
        ipv6.append(Integer.toHexString(Integer.parseInt(hex, HEX_STARING)));
        ipv6.append(":");

        hex = data.substring(14, 16) + data.substring(12, 14);
        ipv6.append(Integer.toHexString(Integer.parseInt(hex, HEX_STARING)));
        ipv6.append(":");

        hex = data.substring(10, 12) + data.substring(8, 10);
        ipv6.append(Integer.toHexString(Integer.parseInt(hex, HEX_STARING)));
        ipv6.append(":");

        hex = data.substring(22, 24) + data.substring(20, 22);
        ipv6.append(Integer.toHexString(Integer.parseInt(hex, HEX_STARING)));
        ipv6.append(":");

        hex = data.substring(18, 20) + data.substring(16, 18);
        ipv6.append(Integer.toHexString(Integer.parseInt(hex, HEX_STARING)));
        ipv6.append(":");

        hex = data.substring(30, 32) + data.substring(28, 30);
        ipv6.append(Integer.toHexString(Integer.parseInt(hex, HEX_STARING)));
        ipv6.append(":");

        hex = data.substring(26, 28) + data.substring(24, 26);
        ipv6.append(Integer.toHexString(Integer.parseInt(hex, HEX_STARING)));

        return ipv6.toString();
    }

    /**
     * Convert a numeric string from Socket status.
     *
     * @param write Socket information text file
     * @param state Socket status.
     * @throws IOException If an I/O error occurs
     */
    private void writeState(final BufferedWriter write, final int state)
            throws IOException {
        switch (state) {
        case VAL_ESTABLISHED:
            write.write(ESTABLISHED);
            break;
        case VAL_SYN_SENT:
            write.write(SYN_SENT);
            break;
        case VAL_SYN_RECV:
            write.write(SYN_RECV);
            break;
        case VAL_FIN_WAIT1:
            write.write(FIN_WAIT1);
            break;
        case VAL_FIN_WAIT2:
            write.write(FIN_WAIT2);
            break;
        case VAL_TIME_WAIT:
            write.write(TIME_WAIT);
            break;
        case VAL_CLOSE:
            write.write(CLOSE);
            break;
        case VAL_CLOSE_WAIT:
            write.write(CLOSE_WAIT);
            break;
        case VAL_LAST_ACK:
            write.write(LAST_ACK);
            break;
        case VAL_LISTEN:
            write.write(LISTEN);
            break;
        case VAL_CLOSING:
            write.write(CLOSING);
            break;
        default:
            write.write("-");
            break;
        }
    }
}
