/*
 * ParserEventHandlerImpl.java
 * Created on 2011/10/13
 * RCSID: $Id:$
 *
 * Copyright (C) 2011-2013 Nippon Telegraph and Telephone Corporation
 *
 */

package jp.co.ntt.oss.heapstats.gui.snapshot;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.nio.channels.FileChannel;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import jp.co.ntt.oss.heapstats.container.ObjectData;
import jp.co.ntt.oss.heapstats.container.SnapShotHeader;
import jp.co.ntt.oss.heapstats.parser.ParserEventHandler;

/**
 * This class is written to the temporary file information file java heap.
 */
public class ParserEventHandlerImpl implements
                                   ParserEventHandler {
    /** OutputStream of temporary files. */
    private ObjectOutputStream stream;

    /** FileChannel of temporary files. */
    private FileChannel streamChannel;

    /** Offset Information. */
    private Map<Date, Long> snapShotOffsets;

    /**
     * This constructor initializes the variable.
     */
    public ParserEventHandlerImpl() {
        stream = null;
        streamChannel = null;
        snapShotOffsets = null;
    }

    /**
     * Performs initialization.
     *
     * @param fname the Output filename.
     * @throws IOException Any of the usual Input/Output related exceptions.
     */
    public final void prepareParse(final String fname) throws IOException {
        FileOutputStream fstream = new FileOutputStream(fname, false);
        streamChannel = fstream.getChannel();
        stream = new ObjectOutputStream(fstream);
        snapShotOffsets = new HashMap<Date, Long>();
    }

    @Override
    public final ParserEventHandler.ParseResult onNewSnapShot(
            final SnapShotHeader header) {
        try {
            snapShotOffsets.put(header.getSnapShotDate(),
                    streamChannel.position());
            stream.writeObject(header);
            stream.reset();
        } catch (Exception e) {
            e.printStackTrace();
            return ParserEventHandler.ParseResult.HEAPSTATS_PARSE_ABORT;
        }
        return ParserEventHandler.ParseResult.HEAPSTATS_PARSE_CONTINUE;
    }

    @Override
    public final ParserEventHandler.ParseResult onEntry(final ObjectData data) {
        try {
            stream.writeObject(data);
            stream.reset();
        } catch (Exception e) {
            e.printStackTrace();
            return ParserEventHandler.ParseResult.HEAPSTATS_PARSE_ABORT;
        }
        return ParserEventHandler.ParseResult.HEAPSTATS_PARSE_CONTINUE;
    }

    /**
     * Close the stream.
     *
     * @throws IOException If an I/O error has occurred.
     */
    public final void endParse() throws IOException {
        if (streamChannel != null) {
            streamChannel.close();
        }
        if (stream != null) {
            stream.close();
        }
    }

    /**
     * Return the offset Information.
     *
     * @return Return the offset Information.
     */
    public final Map<Date, Long> getSnapShotOffsets() {
        return snapShotOffsets;
    }
}
